#ifndef __ERRORHANDLER_H__
#define __ERRORHANDLER_H__

#include <xercesc/sax/HandlerBase.hpp>

XERCES_CPP_NAMESPACE_USE

//! Klasse zur Fehlerbehandlung fuer XML-Parser.
/*!
 * Diese Klasse implementiert Fehlerbehandlungsmethoden, die von XML-Parsern
 * verwendet werden koennen. Neben den klassischen Fehlern des Interfaces
 * ErrorHandler, existiert noch ein weiterer Fehlertyp, der fuer den Test
 * weiterer Constraints verwendet werden kann.
 *
 * @see CINParser
 * @see ConfigParser
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class ErrHandler : public HandlerBase
{
public:

	//! Konstruktor.
	/*!
	 * Setzt initial den Zaehler fuer Fehler auf 0.
	 */
	ErrHandler() : fErrCount(0) {}

	//! Implementierung des ErrorHandler Interfaces.
	void warning(const SAXParseException& exception);
	
	//! Implementierung des ErrorHandler Interfaces.	  
	void error(const SAXParseException& exception);
	
	//! Implementierung des ErrorHandler Interfaces.	  
	void fatalError(const SAXParseException& exception);
    
	//! Ausgabe eines benutzerdefinierten Fehlers.
	void checkError(const SAXParseException& exception);

	//! Ausgabe eines unerwarteten Fehlers.	  
	void unknownError(const SAXParseException& exception);
    
	//! Implementierung des ErrorHandler Interfaces.
	void resetErrors()
	{
		fErrCount = 0;
	}
	
	//! Ueberpruefung auf Fehler.
	inline bool sawErrors() const
	{
		return fErrCount > 0;
	}

	//! Liefert Anzahl der aufgetretenen Fehler,	  
	inline int getErrCount() const
	{
		return fErrCount;
	}
		
private:

	//! Zaehler fuer aufgetretene Fehler.
	int fErrCount;
};

#endif
