#ifndef __MACROS_H__
#define __MACROS_H__

#include <config.h>
#include <sstream>
#include <stdio.h>

#define COMPILE_WITH_MRIP HAVE_MPI && ENABLE_MRIP

#define TICK 0

#define snull ""
#define pnull 0

#define _CONF_FILE_			"CONF_FILE"

#define _XML_PROCESSING_	"XML_PROCESSING"
#define _XML_SCHEMA_		"XML_SCHEMA"
#define _XML_NAMESPACE_		"XML_NAMESPACE"
#define _XML_STOPONERROR_	"XML_STOPONERROR"
#define _XML_INPUT_			"XML_INPUT"

#define _PROCESSING_MIN_	"minimal"
#define _PROCESSING_EXT_	"extended"
#define _PROCESSING_SCHEMA_ "schema"

#define _SHOW_USAGE_		"SHOW_USAGE"
#define _SHOW_PLUGINS_		"SHOW_PLUGINS"
#define _SHOW_VERBOSE_		"SHOW_VERBOSE"
#define _SHOW_PROGRESS_		"SHOW_PROGRESS"
#define _SHOW_TIME_			"SHOW_TIME"

#define _ENGINE_BINARY_		"ENGINE_BINARY"
#define _NICE_LEVEL_		"NICE_LEVEL"			
#define _ENGINES_			"ENGINES"

#define _PRECISION_			"PRECISION"
#define _CONFIDENCE_		"CONFIDENCE"
#define _REPETITIONS_		"REPETITIONS"

#define _DATA_FILE_			"DATA_FILE"
#define _PLOT_FILE_			"PLOT_FILE"

#define _PRNG_ID_			"PRNG_ID"
#define _PRNG_STREAM_		"PRNG_STREAM"

#define _ANALYSIS_ID_		"ANALYSIS_ID"

#define _UNDEFINED_ "-"


#define MSG(WHAT, MESSAGE) \
{ \
	std::stringstream ss; \
	ss << MESSAGE; \
	printf("%s[%d] (%10s, %d)\t[%10s ]\n\tMessage: %s\n", WHAT, TICK, __FILE__, __LINE__, __FUNCTION__, ss.str().c_str()); \
}

#define _ERROR(MESSAGE, ...) \
	{ \
	MSG("<ERROR>", MESSAGE) \
	exit(1); \
	}
	
#define _WARNING(MESSAGE) \
	MSG("<WARNING>", MESSAGE) \


#ifdef DEBUG
	#define INFO
	#define TRACE
	#define _DEBUG(MESSAGE) \
		MSG("<DEBUG>", MESSAGE)

#else
	#define _DEBUG(MESSAGE) {}
#endif

#ifdef INFO
	#define _INFO(MESSAGE) \
		MSG("<INFO>", MESSAGE)
#else
	#define _INFO(MESSAGE) {}
#endif

#ifdef TRACE
	#define PACKETMOVE
	#define _TRACE(MESSAGE) \
		MSG("<TRACE>", MESSAGE)
#else
	#define _TRACE(MESSAGE) {}
#endif

#ifdef PACKETMOVE
	#include <simulation.h>
	#define _PACKETMOVE(ACTION,PACKET,TRACE) \
	{ \
		std::stringstream action, packet, trace; \
		action << ACTION; \
		packet << PACKET; \
		trace << TRACE; \
		printf("<PACKETMOVE>[%d] %s\n %s\n %s", Tick(), action.str().c_str(), packet.str().c_str(), trace.str().c_str()); \
	}

#else 
	#define _PACKETMOVE(ACTION,PACKET,TRACE) {}
#endif

#endif
