#ifndef __PARSERINTERFACE__
#define __PARSERINTERFACE__

#include <ctype.h>

/* Data type for links in the chain of symbols.      */
struct symrec
{
        char *name;  /* name of symbol                     */
        int type;    /* type of symbol: either VAR or FNCT */
        union {
                double var;           /* value of a VAR          */
                double (*fnctptr)(double);  /* value of a FNCT         */
        } value;
        struct symrec *next;    /* link field              */
};

typedef struct symrec symrec;

enum
{
        MP_ERROR_NONE,       /* No error */
        MP_ERROR_SYNTAX,     /* Syntax error */
        MP_ERROR_OOM,        /* Out of memory */
        MP_ERROR_VARIABLE,   /* Too many variables or not enough variable solutions passed */
        MP_ERROR_DIV0,       /* Division by 0 */
        MP_ERROR_SQRTNEG,    /* Square root of a negative */
        MP_ERROR_RECURSION   /* Recursion */
};



enum
{
        MP_TOKEN_CONSTANT,   /* A constant [1] */
        MP_TOKEN_CONST,      /* Predefined constant [pi] */
        MP_TOKEN_VARIABLE,   /* A variable in variables[] [X] */
        MP_TOKEN_PLUS,       /* Addition [X+Y] */
        MP_TOKEN_MINUS,      /* Subtraction [X-Y] */
        MP_TOKEN_UNARY_MINUS,/* Unary minus [-X] */
        MP_TOKEN_MULT,       /* Multiplication [X*Y] */
        MP_TOKEN_DIV,        /* Division [X/Y] */
        MP_TOKEN_FNCT,       /* Function [sin (X)] */
        MP_TOKEN_FACT,       /* Factorial [X!] */
        MP_TOKEN_POW,        /* Power of n [X^Y] */
        MP_TOKEN_ROOT,        /* Root */
        MP_TOKEN_EQ        /* Root */
};


enum
{
        MP_UNIT_RADIAN,      /* Use radians */
        MP_UNIT_DEGREE       /* Use degrees */
};



struct mp_stack
{
        /* Every one of these needs a token type */
        /* value is always used for constants and used during evaluation for vars */
        /* id is used for variables. e.g. 'x' or 'y' */
        /* symboltab is used for functions and predefined constants */
        short  tokentype;
        double  value;
        char   id;
        symrec *symboltab;
        struct mp_stack *next, *previous;
};



// general parser interface class
class ParserInterface
{
public:
	virtual ~ParserInterface() {};
protected:
	virtual symrec *putsym (char*, int) = 0;
	virtual symrec *getsym (char*) = 0;
};


/* The symbol table: a chain of `struct symrec'.     */
//extern symrec *sym_table;

#endif
