#ifndef __TRAFFIC_DISTRIBUTION__
#define __TRAFFIC_DISTRIBUTION__

#include <utility/factory.h>
#include <utility/xmlelement.h>

//! Zeitliche Verteilung von Paketen.
/*!
 * Diese abstrakte Klasse ist Basis fuer alle zeitlichen Verteilungen von 
 * Paketen an den Quellen. Die zeitliche Verteilung beschreibt, wann eine
 * Quelle ein neues Paket generieren kann.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class TrafficDistribution : public FactoryElement, public XMLElement {
public:
	
	//! Konstruktor.
	/*!
	 * Der Konstruktor initialisiert die Laenge der zu verteilenden Pakete
	 * und den Gueltigkeitsbereich innerhalb des XML-Dokumentes.
	 *
	 * @see XMLElement
	 * 
	 * @param packetSize Die (konstante) Groesse der zu verteilenden Pakete.
	 * @param scope
	 */
    TrafficDistribution(unsigned int packetSize=1, const DOMElement* scope=NULL) 
		: XMLElement(scope), constPacketSize(packetSize) {}
		
	//! Der Klassendestruktor.
    virtual ~TrafficDistribution() {}

	//! Test auf Bereitschaft.
	/*!
	 * Diese Methode wird einmal pro Takt von einem Paketgenerator aufgerufen,
	 * legt fest, ob ein neues Paket generiert werden kann oder nicht. Alle
	 * zeitlichen Verteilungen muessen diese Methode implementieren.
	 * 
	 * @param tick Der aktuelle Takt.
	 * @return true, falls laut Verteilung Paket generiert werden kann.
	 */
    virtual bool ifNowGenerate(unsigned int tick) = 0;

	//! Verteilung zuruecksetzen.
	/*!
	 * Diese Methode versetzt die Paketverteilung in ihren initialen Zustand.
	 */
	virtual void reset() = 0;
	
protected:
	
    //! Laenge der zu generierenden Pakete.
    unsigned int constPacketSize;
};


#endif

