#ifndef __HASHMAP_H__
#define __HASHMAP_H__

#include <string>
#include <map>


//! Hashmap fuer beliebige Typen von Keys und Values.
template <class K, class V>
class Hashmap : public std::map<K, V>
{
public:

	//! Suche nach Eintrag.
	/*!
	 *
	 * @return Value zu Key oder Default-Instanz fuer Value.
	 */
	const V& getValue(K key) const
	{
		static V value;
		
		typename Hashmap<K,V>::const_iterator it = this->find(key);

		return it == this->end() ? value : it->second;
	}
};

//! Spezialisierung der Hashmap fuer Pointer
template <class K, class V>
class Hashmap<K, V*> : public std::map<K, V*>
{
public:

	//! Suche nach Eintrag.
	/*!
	 *
	 * @return Value zu Key oder NULL.
	 */
	const V* getValue(K key) const
	{
		typename Hashmap<K, V*>::const_iterator it = this->find(key);

		return it == this->end() ? NULL : (*it).second;
	}
};

template <class T>
class StringMap : public Hashmap<std::string, T> {};

template <class T>
class IntMap : public Hashmap<int, T> {};

#endif
