#ifndef __XML_ATTRIBUTE__
#define __XML_ATTRIBUTE__

#include <utility/valuestring.h>

#include <xercesc/dom/DOM.hpp>

XERCES_CPP_NAMESPACE_USE

//! Einbettung von XML-Attributen.
/*!
 * Diese Klasse ermoeglicht die direkte Verwendung von XML-Attributen aus der
 * XML-Beschreibung einer Simulation zu Initialisierung von CINSim-Komponenten.
 * Dazu werden verschiedene Cast-Operatoren fuer den Attributwert zur Verfuegung 
 * gestellt.
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class XMLAttribute : public ValueString
{
public:
	
	//! Konstruktor.
	/*!
	 * Der Konstruktoraufruf initialisiert den verfuegbaren Wert eines Attributes 
	 * anhand des uebergebenen Attribut-Knotens.
	 *
	 * @param attribute Ein Attribut-Knoten eines XML-Elements.
	 */
	XMLAttribute(const DOMAttr* attribute);

	//! Destruktor.
    ~XMLAttribute() {}

	//! Name des Attributes.
	const char* name() const;
	
private:

	//! Attribut-Knoten.
	const DOMAttr* attribute;
};

#endif
