#ifndef __XMLLOAD_H__
#define __XMLLOAD_H__

#include <environment/parser/cinparser.h>


//! Basisklasse zur Initialisierung von CINSim mit XML-Daten.
/*!
 * Diese (abstrakte) Klasse dient als Basis zum Erstellen einer Simulation oder
 * Simulatiossequenz anhand einer XML-Beschreibung. Die XML-Daten werden einmalig
 * geparst und stehen dann als statische Klassenattribute allen Instanzen dieser
 * Klasse zur Verfuegung.
 *
 * @see CINParser
 * @see XmlSimulation
 * @see XmlSequencer
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class XmlLoad
{
public:
    
    //! Konstruktor.
    XmlLoad() {};
  
	//! (Virtueller) Destruktor.
    virtual ~XmlLoad() {};

    //! Laden einer Simulationsbeschreibung.
    /*!
     * Diese Methode initialisiert einmalig einen XML-Parser, liest das XML-File 
	 * mit der Simulationsbeschreibung ein und erzeugt ein DOM-Dokument und 
	 * Schema-Informationen.
	 *
	 * @see init
     */
    virtual void load();
   
protected:
  
    //! Initialisierung einer Struktur mit eingelesenen XML-Daten
    /*!
     * Diese abstrakte Methode initialisiert eine beliebige Struktur mit den
     * Daten des XML-Files einer Simulationsbeschreibung. Die Daten liegen in Form 
	 * eines DOM-Dokumentes vor. Aufruf erfolgt durch die Methode \a load().
	 *
	 * @see load
     */
    virtual void init()=0;

	//! Schema-Informationen fuer geparste XML-Elemente.
	static const SchemaInformation* _schemaInformation;

	//! Geparstes XML-Dokument (DOM-Baum).
	static const DOMDocument* _document;
};

#endif
